@@ -1,20 +1,12 @@
-/*************************************************************************
- *  Copyright (C) 2012 by Volker Lanz <vl@fidra.de>                      *
- *  Copyright (C) 2016 by Andrius Štikonas <andrius@stikonas.eu>         *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2010 Volker Lanz <vl@fidra.de>
+    SPDX-FileCopyrightText: 2012-2018 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2012,2019 Yuri Chornoivan <yurchor@ukr.net>
+    SPDX-FileCopyrightText: 2020 Arnaud Ferraris <arnaud.ferraris@collabora.com>
+    SPDX-FileCopyrightText: 2020 Gaël PORTAY <gael.portay@collabora.com>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "fs/btrfs.h"
 
@@ -43,7 +35,7 @@ FileSystem::CommandSupportType btrfs::m_SetLabel = FileSystem::cmdSupportNone;
 FileSystem::CommandSupportType btrfs::m_UpdateUUID = FileSystem::cmdSupportNone;
 FileSystem::CommandSupportType btrfs::m_GetUUID = FileSystem::cmdSupportNone;
 
-btrfs::btrfs(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QList<FSFeature>& features) :
+btrfs::btrfs(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QVariantMap& features) :
     FileSystem(firstsector, lastsector, sectorsused, label, features, FileSystem::Type::Btrfs)
 {
 }
@@ -74,7 +66,7 @@ void btrfs::init()
             // First line is introductory text, we don't need it
             lines.removeFirst();
 
-            for (auto l : lines) {
+            for (const auto& l: lines) {
                 if (!l.isEmpty())
                     addAvailableFeature(l.split(QStringLiteral(" ")).first());
             }
@@ -148,12 +140,15 @@ bool btrfs::create(Report& report, const QString& deviceNode)
 
     if (!this->features().isEmpty()) {
         QStringList feature_list = QStringList();
-        for (auto f : this->features()) {
-            if (f.type() == FSFeature::Type::Bool) {
-                if (f.bValue())
-                    feature_list << f.name();
-                else
-                    feature_list << (QStringLiteral("^") +  f.name());
+        for (const auto& k : this->features().keys()) {
+	    const auto& v = this->features().value(k);
+            if (v.type() == QVariant::Type::Bool) {
+                if (v.toBool())
+                    feature_list << k;
+		else
+                    feature_list << (QStringLiteral("^") +  k);
+            } else {
+                qWarning() << "Ignoring feature" << k << "of type" << v.type() << "; requires type QVariant::bool.";
             }
         }
         args << QStringLiteral("--features") << feature_list.join(QStringLiteral(","));
