@@ -1,20 +1,12 @@
-/*************************************************************************
- *  Copyright (C) 2008,2009,2011 by Volker Lanz <vl@fidra.de>            *
- *  Copyright (C) 2016-2017 by Andrius Štikonas <andrius@stikonas.eu>    *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008-2011 Volker Lanz <vl@fidra.de>
+    SPDX-FileCopyrightText: 2013-2019 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2015 Teo Mrnjavac <teo@kde.org>
+    SPDX-FileCopyrightText: 2020 Arnaud Ferraris <arnaud.ferraris@collabora.com>
+    SPDX-FileCopyrightText: 2020 Gaël PORTAY <gael.portay@collabora.com>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "fs/fat16.h"
 
@@ -31,7 +23,7 @@
 
 namespace FS
 {
-fat16::fat16(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QList<FSFeature>& features, FileSystem::Type type) :
+fat16::fat16(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QVariantMap& features, FileSystem::Type type) :
     fat12(firstsector, lastsector, sectorsused, label, features, type)
 {
 }
@@ -50,8 +42,8 @@ void fat16::init()
     m_GetUUID = cmdSupportCore;
 
     if (m_Create == cmdSupportFileSystem) {
-        addAvailableFeature(QStringLiteral("sector-size"), FSFeature::Type::Int);
-        addAvailableFeature(QStringLiteral("sectors-per-cluster"), FSFeature::Type::Int);
+        addAvailableFeature(QStringLiteral("sector-size"));
+        addAvailableFeature(QStringLiteral("sectors-per-cluster"));
     }
 }
 
@@ -89,7 +81,7 @@ bool fat16::create(Report& report, const QString& deviceNode)
 
 bool fat16::resize(Report& report, const QString& deviceNode, qint64 length) const
 {
-    ExternalCommand cmd(report, QStringLiteral("fatresize"), { QStringLiteral("--verbose"), QStringLiteral("--size"), QString::number(length), deviceNode });
+    ExternalCommand cmd(report, QStringLiteral("fatresize"), { QStringLiteral("--verbose"), QStringLiteral("--size"), QString::number(length - 1), deviceNode });
     return cmd.run(-1) && cmd.exitCode() == 0;
 }
 
