@@ -1,30 +1,25 @@
-/*************************************************************************
- *  Copyright (C) 2012 by Volker Lanz <vl@fidra.de>                      *
- *  Copyright (C) 2015 by Teo Mrnjavac <teo@kde.org>                     *
- *  Copyright (C) 2016-2018 by Andrius Štikonas <andrius@stikonas.eu>    *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008-2010 Volker Lanz <vl@fidra.de>
+    SPDX-FileCopyrightText: 2012-2018 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2015 Chris Campbell <c.j.campbell@ed.ac.uk>
+    SPDX-FileCopyrightText: 2015-2016 Teo Mrnjavac <teo@kde.org>
+    SPDX-FileCopyrightText: 2016 Chantara Tith <tith.chantara@gmail.com>
+    SPDX-FileCopyrightText: 2017 Pali Rohár <pali.rohar@gmail.com>
+    SPDX-FileCopyrightText: 2017 Adriaan de Groot <groot@kde.org>
+    SPDX-FileCopyrightText: 2018 Caio Jordão Carvalho <caiojcarvalho@gmail.com>
+    SPDX-FileCopyrightText: 2019 Shubham Jangra <aryan100jangid@gmail.com>
+    SPDX-FileCopyrightText: 2020 Arnaud Ferraris <arnaud.ferraris@collabora.com>
+    SPDX-FileCopyrightText: 2020 Gaël PORTAY <gael.portay@collabora.com>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #ifndef KPMCORE_FILESYSTEM_H
 #define KPMCORE_FILESYSTEM_H
 
-
 #include "util/libpartitionmanagerexport.h"
 
-#include "fs/feature.h"
-
+#include <QVariant>
 #include <QList>
 #include <QStringList>
 #include <QString>
@@ -116,7 +111,7 @@ public:
 
 protected:
     FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, FileSystem::Type type);
-    FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QList<FSFeature>& features, FileSystem::Type type);
+    FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, const QVariantMap& features, FileSystem::Type type);
 
 public:
     virtual ~FileSystem();
@@ -271,16 +266,17 @@ public:
     const QString& label() const;
 
     /**< @return the FileSystem's available features */
-    const QList<FSFeature>& availableFeatures() const;
+    const QStringList& availableFeatures() const;
 
     /**< @return the FileSystem's features */
-    const QList<FSFeature>& features() const;
+    const QVariantMap& features() const;
 
-    /**< @param feature the feature to add to the FileSystem */
-    void addFeature(const FSFeature& feature);
+    /**< @param the feature's name to add to the FileSystem */
+    /**< @param the feature's value to add to the FileSystem */
+    void addFeature(const QString& name, const QVariant& value);
 
     /**< @param features the list of features to add to the FileSystem */
-    void addFeatures(const QList<FSFeature>& features);
+    void addFeatures(const QVariantMap& features);
 
     /**< @return the sector size in the underlying Device */
     qint64 sectorSize() const;
@@ -305,8 +301,7 @@ public:
 
 protected:
     static bool findExternal(const QString& cmdName, const QStringList& args = QStringList(), int exptectedCode = 1);
-    void addAvailableFeature(const FSFeature& feature);
-    void addAvailableFeature(const QString& name, FSFeature::Type type = FSFeature::Type::Bool);
+    void addAvailableFeature(const QString& name);
 
     std::unique_ptr<FileSystemPrivate> d;
 };
