@@ -1,21 +1,13 @@
-/*************************************************************************
- *  Copyright (C) 2017 by Andrius Štikonas <andrius@stikonas.eu>         *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2017-2019 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2018 Caio Jordão Carvalho <caiojcarvalho@gmail.com>
+    SPDX-FileCopyrightText: 2020 Gaël PORTAY <gael.portay@collabora.com>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "plugins/sfdisk/sfdiskpartitiontable.h"
+#include "plugins/sfdisk/sfdiskgptattributes.h"
 
 #include "backend/corebackend.h"
 #include "backend/corebackendmanager.h"
@@ -213,9 +205,54 @@ static QLatin1String getPartitionType(FileSystem::Type t, PartitionTable::TableT
     return QLatin1String();
 }
 
+bool SfdiskPartitionTable::setPartitionLabel(Report& report, const Partition& partition, const QString& label)
+{
+    if (label.isEmpty())
+        return true;
+    ExternalCommand sfdiskCommand(report, QStringLiteral("sfdisk"), { QStringLiteral("--part-label"), m_device->deviceNode(), QString::number(partition.number()),
+                label } );
+    return sfdiskCommand.run(-1) && sfdiskCommand.exitCode() == 0;
+}
+
+QString SfdiskPartitionTable::getPartitionUUID(Report& report, const Partition& partition)
+{
+    ExternalCommand sfdiskCommand(report, QStringLiteral("sfdisk"), { QStringLiteral("--list"), QStringLiteral("--output"), QStringLiteral("Device,UUID"),
+                m_device->deviceNode() });
+    if (sfdiskCommand.run(-1) && sfdiskCommand.exitCode() == 0) {
+        QRegularExpression re(m_device->deviceNode() + QString::number(partition.number()) + QStringLiteral(" +(.+)"));
+        QRegularExpressionMatch rem = re.match(sfdiskCommand.output());
+
+        if (rem.hasMatch())
+            return rem.captured(1);
+    }
+
+    return QString();
+}
+
+bool SfdiskPartitionTable::setPartitionUUID(Report& report, const Partition& partition, const QString& uuid)
+{
+    if (uuid.isEmpty())
+        return true;
+    ExternalCommand sfdiskCommand(report, QStringLiteral("sfdisk"), { QStringLiteral("--part-uuid"), m_device->deviceNode(), QString::number(partition.number()),
+                uuid } );
+    return sfdiskCommand.run(-1) && sfdiskCommand.exitCode() == 0;
+}
+
+bool SfdiskPartitionTable::setPartitionAttributes(Report& report, const Partition& partition, quint64 attrs)
+{
+    QStringList attributes = SfdiskGptAttributes::toStringList(attrs);
+    if (attributes.isEmpty())
+        return true;
+    ExternalCommand sfdiskCommand(report, QStringLiteral("sfdisk"), { QStringLiteral("--part-attrs"), m_device->deviceNode(), QString::number(partition.number()),
+                attributes.join(QStringLiteral(",")) } );
+    return sfdiskCommand.run(-1) && sfdiskCommand.exitCode() == 0;
+}
+
 bool SfdiskPartitionTable::setPartitionSystemType(Report& report, const Partition& partition)
 {
-    QString partitionType = getPartitionType(partition.fileSystem().type(), m_device->partitionTable()->type());
+    QString partitionType = partition.type();
+    if (partitionType.isEmpty())
+        partitionType = getPartitionType(partition.fileSystem().type(), m_device->partitionTable()->type());
     if (partitionType.isEmpty())
         return true;
     ExternalCommand sfdiskCommand(report, QStringLiteral("sfdisk"), { QStringLiteral("--part-type"), m_device->deviceNode(), QString::number(partition.number()),
